open MyrtheAST
open Mv

let repr : value -> int = function
  | Bool true -> 1
  | Bool false -> 0
  | Int i -> i

     
let labelnumber = ref 0

(* crée un nouveau label *)
let newlabel () = 
  let s = "L" ^ (string_of_int !labelnumber) in
     labelnumber := !labelnumber + 1; s

(* traduit une instruction avec ou sans label vers une chaîne
   de caractères en format Krakatau *)
let instrlabel_to_string (x:instrlabel) =
  match x with
    y,z ->
      let s1 = match y with None -> "" | Some l -> l^": " in
      let s2 = match z with
	  Nop -> "nop"
	| Goto s -> "goto " ^ s
	| Bipush i -> "bipush " ^ (string_of_int i)
	| Sipush i -> "sipush " ^ (string_of_int i)
	| Ldc i -> "ldc " ^ (string_of_int i)
	| Iconst_m1 -> "iconst_m1"
	| Iconst_0 -> "iconst_0"
	| Iconst_1 -> "iconst_1"
	| Iconst_2 -> "iconst_2"
	| Iconst_3 -> "iconst_3"
	| Iconst_4 -> "iconst_4"
	| Iconst_5 -> "iconst_5"
	| Iadd -> "iadd"
	| If_icmpeq s -> "if_icmpeq " ^ s
      in
      "   " ^ s1 ^ s2 ^ "\n"

(* traduit une liste d'instructions avec label ou pas vers une chaîne de caractères *)
let list_instrlabel_to_string l =
  let aux x y = x ^ (instrlabel_to_string y) in
  List.fold_left aux "" l 
 
(* traduit une expression Myrthe vers une liste d'instructions avec label ou pas *)
let rec compil = function
  | Const v ->
     let x = repr v in 
     (match x with
       -1 -> [None, Iconst_m1]
     | 0 -> [None, Iconst_0]
     | 1 -> [None, Iconst_1]
     | 2 -> [None, Iconst_2]
     | 3 -> [None, Iconst_3]
     | 4 -> [None, Iconst_4]
     | 5 -> [None, Iconst_5]
     | x when (-128 <= x) && (x < 128) -> [None, Bipush x]
     | x when (-32768 <= x) && (x < 32768) -> [None, Sipush x]
     | _ -> [None, Ldc x])

  | Unop (Succ, e) ->
     (compil e) @ [None, Iconst_1; None, Iadd]

  | Binop (Plus, e1, e2) ->
     (compil e1) @ (compil e2) @ [None, Iadd]
   
  | Binop (Eq, e1, e2) ->      (* on compile e1, ensuite e2 et on compare *)
     let l1 = newlabel () in   (* les deux, résultat 1 si égaux, sinon 0  *)
     let l2 = newlabel () in
     (compil e1) @ (compil e2) @
       [None, (If_icmpeq  l1); None, Iconst_0; None, (Goto l2);
        (Some l1), Iconst_1; (Some l2), Nop]
       
  | _ -> failwith "Students, this is your job !";;

(* produit une chaîne de caractère contenant du JBC pour l'expression *)
let compilfull expr =
  ".version 49 0 
.class super Myrthe 
.super java/lang/Object 

.method public static expression : ()I 
    .code stack 10 locals 10
"
  ^
    (list_instrlabel_to_string (compil expr ))
  ^
"   ireturn
    .end code 
.end method 
.end class
" 
