{
open Lexing
open Parser

exception SyntaxError of string

}

let int = '0' | '-'? ['1'-'9'] ['0'-'9']*
let var_id = ['a'-'z'] ['a'-'z' 'A'-'Z' '0'-'9' '_']*
let blank = [' ' '\t']+
let line = '\r' | '\n' | "\r\n"

rule read =
  parse
  | blank       { read lexbuf }
  | line 	{ read lexbuf }
  | int		{ INT (int_of_string (Lexing.lexeme lexbuf)) }
  | "true"	{ TRUE }
  | "false"	{ FALSE }
  | "if"	{ IF }
  | "then"	{ THEN }
  | "else"	{ ELSE }
  | "not"	{ NOT }
  | "++"        { SUCC }
  | '+'		{ PLUS }
  | '*'		{ TIMES }
  | "&&"	{ AND }
  | "||"	{ OR }
  | '='		{ EQ }
  | "<="	{ LEQ }
  | "("		{ LPAR }
  | ")"		{ RPAR }
  | "let"	{ LET }
  | "in"	{ IN }
  | var_id	{ STRING (Lexing.lexeme lexbuf) }
  | eof		{ EOF }
  | _ 		{  raise (SyntaxError ("Unexpected char: "
    	       	     		      ^(Lexing.lexeme lexbuf))) }
