open Lexer
open Lexing
open Parser
open Compile
open MyrtheAST

let print_bool b = print_string (string_of_bool b)
let print_value = function
    Int i -> print_int i
  | Bool b -> print_bool b
and pr_unop = function
    Not -> print_string "not"
  | Succ -> print_string "succ"
and pr_binop = function
    Plus -> print_string "+"
  | Times -> print_string "*"
  | Et -> print_string "and"
  | Ou -> print_string "or"
  | Eq -> print_string "="
  | Leq -> print_string "<="

let rec print_expr = function
  | Const v -> print_value v
  | Unop (op,e) -> pr_unop op; print_string " "; print_expr e
  | Binop (op,e1,e2) -> print_string "("; print_expr e1; print_string " ";
                        pr_binop op; print_string " ";
                        print_expr e2; print_string ")"
  | If (e1,e2,e3) -> print_string "if ";
                       print_expr e1; 
                       print_string " then "; 
                       print_expr e2; 
                       print_string " else "; 
                       print_expr e3
  | Var v -> print_string v
  | Let(v,e1,e2) -> print_string ("let " ^v^" = ");
                    print_expr e1;
                    print_string " in ";
                    print_expr e2
                    
let parse = Parser.fullexpr Lexer.read

let parse_file f =
  let fin = open_in f in
  let lexbuf = Lexing.from_channel fin in
  let p = parse lexbuf in
  close_in fin;
  p

(* tester lexeur et parseur 
   let _ = print_expr (parse_file Sys.argv.(1)); print_string("\n")  *)
    
let _ = print_string (compilfull (parse_file Sys.argv.(1)))  
    
