%{
open MyrtheAST
%}

%token <int> INT
%token <string> STRING
%token TRUE FALSE
%token IF THEN ELSE
%token LET IN
%token NOT SUCC
%token TIMES PLUS AND OR LEQ EQ
%token LPAR RPAR
%token EOF
%start <MyrtheAST.expression> fullexpr

%nonassoc IN ELSE
%nonassoc EQ LEQ
%left OR
%left AND
%left PLUS
%left TIMES
%left SUCC
%left NOT
%%

fullexpr: e = exp; EOF {e}

exp:
  | v = value
      {Const v}
  | op = unop; e = exp
      {Unop(op,e)}
  | e1 = exp; op=binop; e2 = exp
      {Binop(op,e1,e2)}
  | IF; e1 = exp; THEN; e2 = exp; ELSE; e3 = exp
      {If(e1,e2,e3)}
  | id = STRING
      {Var id}
  | LET; id = STRING; EQ;
    e1 = exp; IN; e2 = exp
      {Let(id,e1,e2)}
  | LPAR; e = exp; RPAR
      {e}  ;

%inline unop:
  | NOT        {Not}
  | SUCC       {Succ}  ;
  
%inline binop:
  | PLUS       {Plus}
  | TIMES      {Times}
  | AND	       {Et}
  | OR	       {Ou}
  | LEQ	       {Leq}
  | EQ	       {Eq}  ;

value:
  | i = INT    {Int i}
  | TRUE       {Bool true}
  | FALSE      {Bool false}  ;

